@@ -131,6 +131,31 @@ static void reset_cursor(zword win)
 } /* reset_cursor */
 
 
+/*
+ * amiga_screen_model
+ *
+ * Check if the Amiga screen model should be used, required for
+ * some Infocom games.
+ *
+ */
+static bool amiga_screen_model (void)
+{
+	if (z_header.interpreter_number == INTERP_AMIGA) {
+		switch (story_id) {
+		case BEYOND_ZORK:
+		case ZORK_ZERO:
+		case SHOGUN:
+		case ARTHUR:
+		case JOURNEY:
+			return TRUE;
+		default:
+			break;
+		}
+	}
+	return FALSE;
+}/* amiga_screen_model */
+
+
 /*
  * reset_screen
  *
@@ -555,15 +580,13 @@ void erase_window(zword win)
 	zword y = wp[win].y_pos;
 	zword x = wp[win].x_pos;
 
-	if (z_header.version == V6 && win != cwin
-	    && z_header.interpreter_number != INTERP_AMIGA)
+	if (z_header.version == V6 && win != cwin && !amiga_screen_model())
 		os_set_colour(lo(wp[win].colour), hi(wp[win].colour));
 
 	os_erase_area(y,
 		      x, y + wp[win].y_size - 1, x + wp[win].x_size - 1, win);
 
-	if (z_header.version == V6 && win != cwin
-	    && z_header.interpreter_number != INTERP_AMIGA)
+	if (z_header.version == V6 && win != cwin && !amiga_screen_model())
 		os_set_colour(lo(cwp->colour), hi(cwp->colour));
 
 	reset_cursor(win);
@@ -1030,10 +1053,19 @@ void z_get_wind_prop(void)
 {
 	flush_buffer();
 
-	if (zargs[1] >= 16)
+	if (zargs[1] < 16)
+		store(((zword *) (wp + winarg0()))[zargs[1]]);
+	else if (zargs[1] == 16)
+		store (os_to_true_colour(lo (wp [winarg0 ()].colour)));
+	else if (zargs[1] == 17) {
+		zword bg = hi (wp [winarg0 ()].colour);
+		if (bg == TRANSPARENT_COLOUR)
+			store ((zword) -4);
+		else
+			store (os_to_true_colour (bg));
+	} else
 		runtime_error(ERR_ILL_WIN_PROP);
 
-	store(((zword *) (wp + winarg0()))[zargs[1]]);
 } /* z_get_wind_prop */
 
 
@@ -1215,7 +1247,7 @@ void z_scroll_window(void)
 	flush_buffer();
 
 	/* Use the correct set of colours when scrolling the window */
-	if (win != cwin && z_header.interpreter_number != INTERP_AMIGA)
+	if (win != cwin && !amiga_screen_model())
 		os_set_colour(lo(wp[win].colour), hi(wp[win].colour));
 
 	y = wp[win].y_pos;
@@ -1226,7 +1258,7 @@ void z_scroll_window(void)
 		       y + wp[win].y_size - 1,
 		       x + wp[win].x_size - 1, (short)zargs[1]);
 
-	if (win != cwin && z_header.interpreter_number != INTERP_AMIGA)
+	if (win != cwin && !amiga_screen_model ())
 		os_set_colour(lo(cwp->colour), hi(cwp->colour));
 } /* z_scroll_window */
 
@@ -1260,10 +1292,16 @@ void z_set_colour(void)
 
 	if (fg == 1)		/* colour 1 is the system default colour */
 		fg = z_header.default_foreground;
+
 	if (bg == 1)
 		bg = z_header.default_background;
 
-	if (z_header.version == V6 && z_header.interpreter_number == INTERP_AMIGA)
+	if (fg == TRANSPARENT_COLOUR)
+		fg = lo (wp[win].colour);
+	if (bg == TRANSPARENT_COLOUR && !(z_header.x_flags & TRANSPARENT_FLAG))
+		bg = hi (wp[win].colour);
+
+	if (z_header.version == V6 && !amiga_screen_model())
 		/* Changing colours of window 0 affects the entire screen */
 		if (win == 0) {
 			int i;
@@ -1291,6 +1329,73 @@ void z_set_colour(void)
 } /* z_set_colour */
 
 
+/*
+ * z_set_true_colour, set the foreground and background colours
+ * to specific RGB colour values.
+ *
+ * zargs[0] = foreground colour
+ * zargs[1] = background colour
+ * zargs[2] = window (-3 is the current one, optional)
+ *
+ */
+void z_set_true_colour (void)
+{
+	zword win = (z_header.version == V6) ? winarg2() : 0;
+	zword true_fg = zargs[0];
+	zword true_bg = zargs[1];
+
+	zword fg = 0;
+	zword bg = 0;
+
+	flush_buffer();
+
+	switch ((short) true_fg) {
+	case -1:	/* colour -1 is the system default colour */
+		fg = z_header.default_foreground;
+		break;
+	case -2:	/* colour -2 means keep current colour */
+		fg = lo(wp[win].colour);
+		break;
+	case -3:	/* colour -3 is the colour at the cursor */
+		fg = os_peek_colour();
+		break;
+	case -4:
+		fg = lo (wp[win].colour);
+		break;
+	default:
+		fg = os_from_true_colour(true_fg);
+		break;
+	}
+
+	switch ((short) true_bg) {
+	case -1: /* colour -1 is the system default colour */
+		bg = z_header.default_background;
+		break;
+	case -2: /* colour -2 means keep current colour */
+ 		bg = hi (wp[win].colour);
+		break;
+	case -3: /* colour -3 is the colour at the cursor */
+		bg = os_peek_colour();
+		break;
+	case -4: /* colour -4 means transparent */
+		if (z_header.x_flags & TRANSPARENT_FLAG)
+			bg = TRANSPARENT_COLOUR;
+		else
+			bg = hi(wp[win].colour);
+		break;
+	default:
+		bg = os_from_true_colour(true_bg);
+		break;
+	}
+
+	wp[win].colour = (bg << 8) | fg;
+
+	if (win == cwin || z_header.version != V6)
+		os_set_colour(fg, bg);
+
+}/* z_set_true_colour */
+
+
 /*
  * z_set_font, set the font for text output and store the previous font.
  *
@@ -1504,23 +1609,16 @@ void z_show_status(void)
 	print_object(global0);
 
 	/* A header flag tells us whether we have to display the current
-	   time or the score/moves information */
+	 * time or the score/moves information.
+	 */
 	if (z_header.config & CONFIG_TIME) {	/* print hours and minutes */
-		zword hours;
+		zword hours = global1;
 
-		/* Cutthroats puts 111 in the hour when the PC is not
-		 * wearing a watch.  Most (all?) Infocom's interpreters
-		 * save for the Amiga one would handle 24-hour conversion
-		 * to am/pm notation by subtracting 12 from the hour if it's
-		 * greater than 12.  So, by putting 111 in there,
-		 * subtracting 12 results in 99.  The Amiga interpreter and
-		 * all modern interpreters do the conversion correctly.
-		 * The result is 3.
-		 */
-		if (story_id == CUTTHROATS && global1 >= 12)
-			hours = 99;
-		else
-			hours = (global1 + 11) % 12 + 1;
+		/* Cutthroats depends on this simple but "wrong" way. */
+		if (hours == 0)
+			hours = 24;
+		if (hours > 12)
+			hours -= 12;
 
 		pad_status_line(brief ? 15 : 20);
 		print_string("Time: ");
